

/* hashblck.h */


/*------------------------------------------------------------------------
 * Module for management of a single hash block and a hash block list
 *------------------------------------------------------------------------
 */


/*
 * A single HASHBLCK is one block of a linear linked list of HASHBLCKs that
 * build a hash list. A hash list (or bucket) contains solitaire fields with
 * an identical HashKey to speed up the field search.
 *
 * A HASHBLCK can contain a defined number of entries. It includes a FIELDHSH
 * created with this number of entries for the data. The formats of the data
 * and the statistic are strictly separated from the hash structure and are
 * not part of this module.
 *
 * To collect all relevant constants together in one location, the number of
 * hash entries is defined in field.h.
 */


/*----------------------------------------------------------------------*/

#ifndef HASHBLCK_H
#define HASHBLCK_H

/*----------------------------------------------------------------------*/

#include "mytypes.h"
#include "field.h"

/*----------------------------------------------------------------------*/


typedef struct s_HASHBLCK *tpsHASHBLCK;
typedef tpsHASHBLCK       *tppsHASHBLCK;

/*---------------------------------------------*/

tVOID        vHashBlckOpen           (tVOID);
tVOID        vHashBlckClose          (tVOID);
tVOID        vHashBlckMemEmpty       (tpCOUNT pcnSizeof, tpCOUNT pcnBlocks);

tpsHASHBLCK  psHashBlckCreate        (tCOUNT cnHashKey);
tpsHASHBLCK  psHashBlckDestroy       (tpsHASHBLCK psBlck);
tpsHASHBLCK  psHashBlckMemUsage      (tpsHASHBLCK psBlck, tpCOUNT pcnSizeof, tpCOUNT pcnBlocks);

tppsHASHBLCK ppsHashBlckListDestroy  (tppsHASHBLCK ppsList);
tppsHASHBLCK ppsHashBlckListMemUsage (tppsHASHBLCK ppsList, tpCOUNT pcnSizeof, tpCOUNT pcnBlocks);

tVOID        vHashBlckListInsert     (tppsHASHBLCK ppsList, tpsFIELDVAL psVal);

tVOID        vHashBlckListConcat     (tppsHASHBLCK ppsList, tppsHASHBLCK ppsAdd);
tBOOL        bHashBlckListLoop       (tppsHASHBLCK ppsList, tpsFIELDVAL psVal);

tCOUNT       cnHashBlckListEntries   (tppsHASHBLCK ppsList);


/*----------------------------------------------------------------------*/

#endif
