

/* mymemory.c */


/*------------------------------------------------------------------------
 * my-library for the basic dynamic memory allocation
 *------------------------------------------------------------------------
 * This version of mymemory is part of the Solitaire Brute Force tool.
 * Copyright (C) 2001 Holger Hasselbach <H.Hasselbach@galad.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *------------------------------------------------------------------------
 */


/*----------------------------------------------------------------------*/

#include "mymemory.h"

#include <stdlib.h>
#include <stdio.h>

/*----------------------------------------------------------------------*/


/*-----------------------------------------------
 * Allocation counters
 *-----------------------------------------------
 */

static tCOUNT cn_Memory_AllocCount = 0;
static tCOUNT cn_Memory_FreeCount  = 0;


/*-----------------------------------------------
 * Allocate memory
 *-----------------------------------------------
 * If there isn't enough (malloc returns NULL),
 * the program is terminated with a critical error.
 * This is okay for the solitaire analysis, it might
 * be considered harmful in other circumstances
 * regarding loss of unsaved data.
 * cnSize: Size of memory in bytes
 * 0 is a valid value and returns NULL.
 *-----------------------------------------------
 */

tpVOID pvMemoryAlloc (tCOUNT cnSize)
{
    tpVOID pvMem = NULL;

    if (cnSize)
      {
        pvMem = malloc(cnSize);
        if (!pvMem)
          {
            fprintf(stderr,"\n\n*** MemoryAlloc error, %lu required!\n\n",(unsigned long)cnSize);
            exit(EXIT_FAILURE);
          }
        cn_Memory_AllocCount++;
      }

    return (pvMem);
}


/*-----------------------------------------------
 * Free memory
 *-----------------------------------------------
 * NULL is valid for pvMem. Return is always NULL.
 *-----------------------------------------------
 */

tpVOID pvMemoryFree (tpVOID pvMem)
{
    if (pvMem)
      {
        free(pvMem);
        cn_Memory_FreeCount++;
      }

    return (NULL);
}


/*-----------------------------------------------
 * Readout alloc- and freecount
 *-----------------------------------------------
 */

tCOUNT cnMemoryAllocCount (tVOID)
{
    return (cn_Memory_AllocCount);
}


tCOUNT cnMemoryFreeCount (tVOID)
{
    return (cn_Memory_FreeCount);
}


/*----------------------------------------------------------------------*/
