

/* statsymm.c */


/*------------------------------------------------------------------------
 * Solitaire field statistics: Symmetry
 *------------------------------------------------------------------------
 * This version of statsymm is part of the Solitaire Brute Force tool.
 * Copyright (C) 2001 Holger Hasselbach <H.Hasselbach@galad.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *------------------------------------------------------------------------
 */


/*----------------------------------------------------------------------*/

#include "statsymm.h"
#include "mymemory.h"
#include "stattree.h"

#include <assert.h>

/*----------------------------------------------------------------------*/


struct s_STATSYMM
{
    tpsSTATTREE psTreeAll;
    tpsSTATTREE psTreeNrm;
};

typedef struct s_STATSYMM tsSTATSYMM;


/*----------------------------------------------------------------------*/


/*-----------------------------------------------
 * Create a new STATSYMM
 *-----------------------------------------------
 */

tpsSTATSYMM psStatSymmCreate (tVOID)
{
    tpsSTATSYMM psSymm;

    psSymm = pvMemoryAllocObject(sSTATSYMM);

    psSymm->psTreeAll = psStatTreeCreate();
    psSymm->psTreeNrm = psStatTreeCreate();

    return (psSymm);
}


/*-----------------------------------------------
 * Destroy a STATSYMM
 *-----------------------------------------------
 */

tpsSTATSYMM psStatSymmDestroy (tpsSTATSYMM psSymm)
{
    if (psSymm)
      {
        psStatTreeDestroy(psSymm->psTreeAll);
        psStatTreeDestroy(psSymm->psTreeNrm);

        pvMemoryFree(psSymm);
      }

    return (NULL);
}


/*-----------------------------------------------
 * Memory statistics
 *-----------------------------------------------
 */

tpsSTATSYMM psStatSymmMemUsage (tpsSTATSYMM psSymm, tpCOUNT pcnSizeof, tpCOUNT pcnBlocks)
{
    if (psSymm)
      {
        psStatTreeMemUsage(psSymm->psTreeAll,pcnSizeof,pcnBlocks);
        psStatTreeMemUsage(psSymm->psTreeNrm,pcnSizeof,pcnBlocks);

        *pcnSizeof += cnMemorySizeofObject(sSTATSYMM);
        *pcnBlocks += 1;
      }

    return (psSymm);
}


/*-----------------------------------------------
 * Add a field statistic
 *-----------------------------------------------
 */

tVOID vStatSymmAddField (tpsSTATSYMM psSymm, tpsFIELDSTT psStt)
{
    vStatTreeAddField(psSymm->psTreeAll,psStt,TRUE );
    vStatTreeAddField(psSymm->psTreeNrm,psStt,FALSE);
}


/*-----------------------------------------------
 * Add a STATSYMM to another STATSYMM
 *-----------------------------------------------
 */

tVOID vStatSymmAddObject (tpsSTATSYMM psSymm, tpsSTATSYMM psAdd)
{
    vStatTreeAddObject(psSymm->psTreeAll,psAdd->psTreeAll);
    vStatTreeAddObject(psSymm->psTreeNrm,psAdd->psTreeNrm);
}


/*-----------------------------------------------
 * Calculate statistic
 *-----------------------------------------------
 */

tVOID vStatSymmCalcWhole (tpsSTATSYMM psSymm)
{
    vStatTreeCalcWhole(psSymm->psTreeAll);
    vStatTreeCalcWhole(psSymm->psTreeNrm);
}


/*-----------------------------------------------
 * Test if there is any pattern at all
 *-----------------------------------------------
 */

tBOOL bStatSymmIsEmpty (tpsSTATSYMM psSymm)
{
    return ( bStatTreeIsEmpty(psSymm->psTreeAll) );
}


/*-----------------------------------------------
 * Get the maximum number of moves
 *-----------------------------------------------
 */

tCOUNT cnStatSymmMaxMoves (tpsSTATSYMM psSymm)
{
    return ( cnStatTreeMaxMoves(psSymm->psTreeAll) );
}


/*----------------------------------------------------------------------*/


/*-----------------------------------------------
 * Store parameter into table
 *-----------------------------------------------
 * NULL is not valid for pzContent.
 * Content qualifier:
 * "A": TreeAll
 * "N": TreeNrm
 *-----------------------------------------------
 */

tVOID vStatSymmTabSet (tpsSTATSYMM psSymm, tpsSTATTABS psTabs, tPSZc pzContent)
{
    switch (*pzContent)
      {
      case 'A' :
        vStatTabsStyleEntryAll(psTabs);
        vStatTreeTabSet(psSymm->psTreeAll,psTabs,pzContent + 1);
        break;
      case 'N' :
        vStatTabsStyleEntryNrm(psTabs);
        vStatTreeTabSet(psSymm->psTreeNrm,psTabs,pzContent + 1);
        break;

      default : assert(FALSE);
      }
}


/*----------------------------------------------------------------------*/
