

/* wordcnt.h */


/*------------------------------------------------------------------------
 * Module for big integer arithmetics based on arrays of UWORDs
 *------------------------------------------------------------------------
 */


/*
 * This module implements big integer arithmetics (addition only) in the form
 * of dynamic counters. The counters automatically modify their bit size when
 * the value overflows the actual boundary.
 *
 * The structure is optimized for the solitaire statistics and the hash block
 * structure. A number of counters of identical size is stored in an array
 * of UWORDs. When one counter overflows, the size is adapted for all
 * counters in the array.
 *
 * Index checking is done by assert.
 */


/*----------------------------------------------------------------------*/

#ifndef WORDCNT_H
#define WORDCNT_H

/*----------------------------------------------------------------------*/

#include "mytypes.h"

/*----------------------------------------------------------------------*/


typedef struct s_WORDCNT *tpsWORDCNT;
typedef tpsWORDCNT       *tppsWORDCNT;

/*---------------------------------------------*/

struct s_WORDCNT
{
    tCOUNT  cnWords;     /* Number of words per counter        */
    tCOUNT  cnCounters;  /* Number of counters                 */

    tpUWORD puwInts;     /* Counters, cnWords*cnCounters WORDs */
};

typedef struct s_WORDCNT tsWORDCNT;

/*---------------------------------------------*/

tpsWORDCNT psWordCntInit     (tpsWORDCNT psCnt, tCOUNT cnCounters);
tpsWORDCNT psWordCntExit     (tpsWORDCNT psCnt);
tpsWORDCNT psWordCntMemInner (tpsWORDCNT psCnt, tpCOUNT pcnSizeof, tpCOUNT pcnBlocks);

tpsWORDCNT psWordCntCreate   (tCOUNT cnCounters);
tpsWORDCNT psWordCntDestroy  (tpsWORDCNT psCnt);
tpsWORDCNT psWordCntMemUsage (tpsWORDCNT psCnt, tpCOUNT pcnSizeof, tpCOUNT pcnBlocks);

tVOID      vWordCntInit0     (tpsWORDCNT psCnt, tCOUNT cnIndex);
tVOID      vWordCntInit1     (tpsWORDCNT psCnt, tCOUNT cnIndex);
tVOID      vWordCntInit0All  (tpsWORDCNT psCnt);
tBOOL      bWordCntIs0       (tpsWORDCNT psCnt, tCOUNT cnIndex);
tCOUNT     cnWordCntUsed     (tpsWORDCNT psCnt, tCOUNT cnIndex);

tVOID      vWordCntCopy      (tpsWORDCNT psCnt, tCOUNT cnICnt, tpsWORDCNT psSrc, tCOUNT cnISrc);

tVOID      vWordCntAdd       (tpsWORDCNT psCnt, tCOUNT cnICnt, tpsWORDCNT psAdd, tCOUNT cnIAdd);
tVOID      vWordCntAddMulti  (tpsWORDCNT psCnt, tCOUNT cnICnt, tpsWORDCNT psAdd, tCOUNT cnIAdd, tUWORD uwFactor);
tVOID      vWordCntAddWord   (tpsWORDCNT psCnt, tCOUNT cnICnt, tUWORD uwAdd);

tpPSZ      ppzWordCntToStr   (tpPSZ ppzStr, tpsWORDCNT psCnt, tCOUNT cnIndex, tUWORD uwBase);


/*----------------------------------------------------------------------*/

#endif
